// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

#include "GLVector.h"

using namespace std;

GLVector::GLVector() : x(0.0f), y(0.0f), z(0.0f) {
}

GLVector::GLVector(const GLVector &v) : x(v.x), y(v.y), z(v.z) {
}

GLVector::GLVector(float inX, float inY, float inZ) : x(inX), y(inY), z(inZ) {
}

GLVector::~GLVector() {
}

float &GLVector::operator[](unsigned int i) {
  return (&x)[i];
}

const float &GLVector::operator [](unsigned int i) const {
  return (&x)[i];
}

void GLVector::normalize() {
  float length = magnitude();
  x /= length;
  y /= length;
  z /= length;
}

float GLVector::magnitude() {
  return sqrt(x*x+y*y+z*z);
}

GLVector GLVector::blend(float f, const GLVector &v1) {
  return GLVector(x + (f*(v1.x - x)), y + (f*(v1.y - y)), z + (f*(v1.z - z)));
}

void GLVector::clear() {
  x = 0.0f;
  y = 0.0f;
  z = 0.0f;
}

void GLVector::set(float inX, float inY, float inZ) {
  x = inX;
  y = inY;
  z = inZ;
}

GLVector &GLVector::operator=(const GLVector &right) {
  x = right.x;
  y = right.y;
  z = right.z;
  return *this;
}

GLVector &GLVector::operator+=(const GLVector &right) {
  x += right.x;
  y += right.y;
  z += right.z;
  return *this;
}

GLVector &GLVector::operator-=(const GLVector &right) {
  x -= right.x;
  y -= right.y;
  z -= right.z;
  return *this;
}

GLVector &GLVector::operator %=(const GLVector &v1) {
  GLVector vt(*this);
  x = vt.y*v1.z - vt.z*v1.y;
  y = vt.z*v1.x - vt.x*v1.z;
  z = vt.x*v1.y - vt.y*v1.x;
  return *this;
}

GLVector &GLVector::operator+=(const float right) {
  x += right;
  y += right;
  z += right;
  return *this;
}

GLVector &GLVector::operator-=(const float right) {
  x -= right;
  y -= right;
  z -= right;
  return *this;
}

GLVector &GLVector::operator*=(const float right) {
  x *= right;
  y *= right;
  z *= right;
  return *this;
}

GLVector &GLVector::operator/=(const float right) {
  x /= right;
  y /= right;
  z /= right;
  return *this;
}

GLVector operator+(const GLVector &left, const GLVector &right) {
  return GLVector(left) += right;
}

GLVector operator-(const GLVector &left, const GLVector &right) {
  return GLVector(left) -= right;
}

ostream &operator<<(ostream &stream, const GLVector &v) {
  stream << v.asString();
  return stream;
}

float operator*(const GLVector &left, const GLVector &right) {
  return left.x*right.x + left.y*right.y + left.z*right.z;
}

GLVector operator%(const GLVector &left, const GLVector &right) {
  return GLVector(left) %= right;
}

GLVector computeNormal(const GLVector &v1, const GLVector &v2, const GLVector &v3, int orientation) {
  GLVector normal;
  normal = ((orientation == FACE_CCW?v3:v1)-v2)%((orientation == FACE_CCW?v1:v3) - v2);
  normal.normalize();
  return normal;
}

GLVector operator+(const GLVector &left, const float x) {
  return GLVector(left) += x;
}

GLVector operator+(const float x, const GLVector &right) {
  return GLVector(right) += x;
}

GLVector operator-(const GLVector &left, const float x) {
  return GLVector(left) -= x;
}

GLVector operator*(const GLVector &left, const float x) {
  return GLVector(left) *= x;
}

GLVector operator*(const float x, const GLVector &right) {
  return GLVector(right) *= x;
}

GLVector operator/(const GLVector &left, const float x) {
  return GLVector(left) /= x;
}

char *GLVector::asString() const {
  char *str = new char[25];
  sprintf(str, "(%f ; %f ; %f)", x, y, z);
  return str;
}

void GLVector::print() const {
  printf("%s\n", asString());
}

