// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

#ifndef _GL_VECTOR_H_
#define _GL_VECTOR_H_

#include <cmath>
#include <iostream>
#include <cstdio>

#define FACE_CCW	0
#define FACE_CW		1

using namespace std;

/**
 * A three-floats vector for use with the OpenGL stuff
 */
class GLVector {
public:
  /**
   * Default constructor
   */
  GLVector();

  /**
   * Copy constructor
   */
  GLVector(const GLVector& v);

  /**
   * Constructor specifying the three floats
   *
   *\param inX the x value to set
   *\param inY the y value to set
   *\param inZ the z value to set
   */
  GLVector(float inX, float inY, float inZ);

  /**
   * Default destructor
   */
  ~GLVector();

  float x;
  float y;
  float z;

  /**
   * Set x, y, z to new values
   *
   *\param inX new x value
   *\param inY new y value
   *\param inZ new z value
   */
  void set(float inX, float inY, float inZ);

  /**
   * Stretch the vector by the float f.
   *
   *\param f value to stretch the vector by
   *\param v1 GLVector to stretch
   *\return new GLVector stretched by f
   */
  GLVector blend(float f, const GLVector &v1);

  /**
   * Normalize the vector, making the magnitude = 1
   */
  void normalize();

  /**
   * Get the vector's magnitude.
   *
   *\return the vector's magnitude
   */
  float magnitude();

  /**
   * Set x, y, z to 0.0f
   */
  void clear();
  
  /**
   * Make the vector accessible by the array operator
   *
   *\
   */
  float &operator[](unsigned int i);

  /**
   * Make the vector accessible by the array operator
   *
   * I don't quite get the difference between this and the above function
   */
  const float &operator[](unsigned int i) const;

  /**
   * Overload the = operator to work with GLVector
   */
  GLVector &operator=(const GLVector &right);

  /**
   * Overload the += operator to work with GLVector
   */
  GLVector &operator+=(const GLVector &right);

  /**
   * Overload the -= operator to work with GLVector
   */
  GLVector &operator-=(const GLVector &right);
  
  /**
   * Overload the += operator to work with GLVector
   */
  GLVector &operator+=(const float right);

  /**
   * Overload the -= operator to work with GLVector
   */
  GLVector &operator-=(const float right);

  /**
   * Overload the *= operator to work with GLVector
   */
  GLVector &operator*=(const float right);

  /**
   * Overload the %= operator to work with GLVector
   */
  GLVector &operator%=(const GLVector &v1);

  /**
   * Overload the /= operator to work with GLVector
   */
  GLVector &operator/=(const float right);

  /**
   * Get a char* of the vector to get a textual representation
   */
  char *asString() const;

  /**
   * Print the vector to std::out
   */
  void print() const;

};

/**
 * Compute the normal vector of the surface designated by two of the three vectors
 * and the orientation
 *
 *\param v1 first vector of the surface
 *\param v2 second vector of the surface
 *\param v3 third vector of the surface
 *\param orientation which of the three vectors to use to get the normal point into
 * the right direction
 */
GLVector computeNormal(const GLVector &v1, const GLVector &v2, const GLVector &v3, 
                       int orientation = FACE_CCW);

/**
 * Overload the << operator to allow writing a GLVector to an outstream
 */
ostream &operator<<(ostream &stream, const GLVector &v);

/**
 * Overload the + operator to allow adding two GLVectors
 */
GLVector operator+(const GLVector &left, const GLVector &right);

/**
 * Overload the - operator to allow subtracting two GLVectors
 */
GLVector operator-(const GLVector &left, const GLVector &right);

/**
 * Overload the * operator to allow multiplying two GLVectors
 */
float operator*(const GLVector &left, const GLVector &right);

/**
 * Overload the % operator to allow modulo with two GLVectors
 */
GLVector operator%(const GLVector &left, const GLVector &right);

/**
 * Overload the + operator to allow adding a GLVector and a float
 */
GLVector operator+(const GLVector &left, const float x);

/**
 * Overload the + operator to allow adding a GLVector and a float
 */
GLVector operator+(const float x, const GLVector &right);

/**
 * Overload the - operator to allow subtracting a float from a GLVector
 */
GLVector operator-(const GLVector &left, const float x);

/**
 * Overload the * operator to allow multiplying a GLVector and a float
 */
GLVector operator*(const GLVector &left, const float x);

/**
 * Overload the * operator to allow multiplying a GLVector and a float
 */
GLVector operator*(const float x, const GLVector &right);

/**
 * Overload the / operator to allow dividing a GLVector by a float
 */
GLVector operator/(const GLVector &left, const float x);

#endif
