// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

#include "arcball.h"

ArcBall::ArcBall(CalVector center, GLfloat radius) :
  qAdjust_(0, 0, 0, 1), vNow_(0, 0, 0), vDown_(0, 0, 0), vFrom_(0, 0, 0), vTo_(0, 0, 0),
  mNow_(CalQuaternion(0, 0, 0, 0)), mDown_(CalQuaternion(0, 0, 0, 0)) 
{
  setDimensions(radius);
  setPosition(center);
  dragging_ = false;
}

CalVector ArcBall::mapToSphere(CalVector p) {
  GLfloat length;
  CalVector v(0, 0, 0);
	
  v.x = (p.x - center_.x) / radius_;
  v.y = (p.y - center_.y) / radius_;

  length = (v.x * v.x) + (v.y * v.y);
  if(length > 1.0f) {
    GLfloat scale = 1.0f/sqrt(length);
    v.x *= scale;
    v.y *= scale;
    v.z = 0.0f;
  } else {
    v.z = sqrt(1 - length);
  }
  return v;
}

void ArcBall::setDimensions(GLfloat radius) {
  radius_ = radius;
}

void ArcBall::setPosition(CalVector position) {
  center_ = position;
}

void ArcBall::mouse(CalVector vNow) {
  vNow_ = vNow;
}

void ArcBall::update() {
  vTo_ = mapToSphere(vNow_);
  if (dragging_) {
    CalVector f(vFrom_), t(vTo_);
    f *= qAdjust_;
    t *= qAdjust_;
    qDrag_ = QuatFromBallPoints(f, t);
    qNow_ = qDown_ * qDrag_;
  }
  CalQuaternion qT(qNow_);
  qT.conjugate();
  mNow_ = qT;
}


CalMatrix ArcBall::value() {
  return mNow_; 
}

CalQuaternion ArcBall::qValue() {
  CalQuaternion qT(qNow_);
  qT.conjugate();
  return qT;
}

void ArcBall::beginDrag() {
  dragging_ = true;
  vDown_ = vNow_;
  vFrom_ = mapToSphere(vDown_);
}

void ArcBall::endDrag() {
  dragging_ = false;
  qDown_ = qNow_;
  mDown_ = mNow_;
}

void ArcBall::setState(CalVector v) {
  double scale = (2*PI)/360;
  v *= scale;
  qNow_ = QuatFromAxisAngle(0, v.x)*QuatFromAxisAngle(1, v.y)*QuatFromAxisAngle(2, v.z);
  qDown_ = qNow_;
}

CalVector ArcBall::getState() {
  double scale = 360/(2*PI);
  CalVector v = QuatToEuler(qNow_);
  v = v * scale;
  return v;
}

void ArcBall::adjustRotation(CalQuaternion q) {
  qAdjust_ = q;
}

void ArcBall::render() {
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glOrtho(-1.0, 1.0, -1.0, 1.0, 1, -100);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glPushAttrib( GL_ENABLE_BIT );
  glDisable( GL_LIGHTING);
  glScalef(radius_, radius_, radius_);
  glLineWidth(1.0f);
  glColor3f(1, 1, 1);
  DrawCircle(1);
  DrawCursor();
  DrawDragArc();
  glPopAttrib();
  glPopMatrix();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();	
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
}

void ArcBall::DrawDragArc() {
  glColor3f(0.8, 0.6, 0.2);
  if (dragging_) {

    DrawAnyArc(vFrom_, vTo_);
  }
}

void ArcBall::DrawCursor() {
  if(dragging_) {
    glColor3f(0.5, 0.5, 1.0);
    glPointSize(12.0f);
    glBegin(GL_POINTS);
    glVertex3f(vTo_.x, vTo_.y, vTo_.z);
    glEnd();	
    glColor3f(0.5, 1.0, 0.5);
    glPointSize(12.0f);
    glBegin(GL_POINTS);
    glVertex3f(vFrom_.x, vFrom_.y, vFrom_.z);
    glEnd();	
  }
}
