// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

/**\file efmath.h
 * Various useful mathematical operations
 *
 *\author Henri Kuuste
 *\author Kai Blin (Documentation)
 */

#ifndef _EF_EFMATH_H_
#define _EF_EFMATH_H_

#include <cal3d/cal3d.h>
#include <GL/gl.h>
#include <cmath>


/**
 * Pi is set to 3.1415926535, as expected
 */
#define PI   3.1415926535

/**
 * Convert a quaternation to a 5x5 matrix
 *
 *\param quat quaternation to convert
 *\param m a 5x5 array of float, to hold the 5x5 matrix
 */
void QuatToMatrix(CalQuaternion quat, float m[4][4]);

/**
 * Apply the quaternation to ?current? matrix
 *
 *\param quat quaternation to apply
 */
void applyQuaternion(CalQuaternion quat);

/**
 * Convert a CalMatrix object into an array-based 5x5 matrix
 *
 *\param cm CalMatrix to convert
 *\param m a 5x5 array to hold the result
 */
void CalMatrixToMatrix(CalMatrix cm, float m[4][4]);

/**
 * Apply a CalMatrix to ?the current GL context?
 */
void applyCalMatrix(CalMatrix m);

/**
 * Convert an Euler CalVector to a CalQuaternation
 *
 *\param v CalVector to convert
 *\return the CalQuaternation computed from the vector
 */
CalQuaternion EulerToQuat(CalVector v);

/**
 * Convert a CalQuaternation to an Euler CalVector
 *
 *\param q1 CalQuaternation to convert
 *\return the Euler vector computed from the quaternation
 */
CalVector QuatToEuler(CalQuaternion q1);

/**
 * Compute a quaternation from an axis and an angle
 *
 *\param axis an int value from 0 - 2 to indicate the axis to use 0 == x, 1 == y, 2 == z
 *\param angle the angle to the axis
 *\return a CalQaternation computed from the axis and angle
 */
CalQuaternion QuatFromAxisAngle(int axis, double angle);

/**
 * I have no clue what a ball point is
 *
 *\todo fix the documentation for this function
 *\param q CalQuaternation to compute the ball points from
 *\param from pointer to CalVector to hold ?from?
 *\param to pointer to CalVector to hold ?to?
 *\bug it should only rotate half the angle
 */
void QuatToBallPoints(CalQuaternion q, CalVector *from, CalVector *to);

/**
 * Compute a CalQuaternation from two points
 *
 *\param from starting point of the quaternation
 *\param to end point of the quaternation
 *\return CalQuaternation computed from the two points
 */
CalQuaternion QuatFromBallPoints(CalVector from, CalVector to);
//CalVector ConstrainToAxis(CalVector p, CalVector axis);

/**
 * Bisect two vectors
 *
 *\todo what does the code do? looks like adding two vectors and
 * normalizing them. Is that called bisection in English?
 *\param v0 first vector to use
 *\param v1 second vector to use
 *\return the ?bisected? vector
 *\bug normalize vectors first
 */
CalVector Bisect(CalVector v0, CalVector v1);

/**
 * Draw an OpenGL arc from one point to another point
 *
 *\param vFrom point to start drawing the arc from
 *\param vTo point to stop drawing the arc
 */
void DrawAnyArc(CalVector vFrom, CalVector vTo);

/**
 * Draw an OpenGL circle
 *\param radius the radius the circle should have
 */
void DrawCircle(GLfloat radius);

#endif
