// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

#include "obj.h"

using namespace std;

Obj::Obj(const std::string file, CalBone *bone) : StaticObject(bone) {
  Obj();
  readFile(file, true);
  readFile(file, false);
}

Obj::Obj() {
  orientation = FACE_CCW;
}

Obj::~Obj() {
  vertices.clear();
  textureCoords.clear();
  normals.clear();
  for(unsigned int i = 0; i < meshes.size(); i++) {
    for(unsigned int j = 0; j < meshes[i].faces.size(); j++) {
      meshes[i].faces[j].vertexIds.clear();
    }
    meshes[i].faces.clear();
  }
  meshes.clear();
  assert(vertices.empty());
  assert(textureCoords.empty());
  assert(normals.empty());
  assert(meshes.empty());
}

void Obj::reserve(unsigned int numVertices, unsigned int numNormals, unsigned int numTexCoords, vector<unsigned int> &numMeshFaces) {
  vertices.reserve(numVertices);
	
  textureCoords.reserve(numTexCoords);

  normals.reserve(numNormals);

  meshes.reserve(numMeshFaces.size());
  meshes.resize(numMeshFaces.size());

  for(unsigned int i = 0; i < numMeshFaces.size(); i++) {
    meshes[i].faces.reserve(numMeshFaces[i]);
    meshes[i].gotFlags = false;
  }
}

bool Obj::readFile(std::string file, bool runDry) {
  ifstream obj(file.c_str(), ios::in);
  if(!obj) {
    cout << "Object file could not be read!" << " filename: " << file << endl;
    return false;
  }

  char line[200];
  unsigned int numVertices = 0, numTexCoords = 0, numNormals = 0;
  vector<unsigned int> numMeshFaces;
  int nMesh = -1;

  while(obj.getline(line, 200, '\n')) {
    switch(line[0]) {
    case 'v':
      switch(line[1]) {
      case ' ':
        if(runDry) {
          numVertices++;
        } else {
          getVertex(string(line+2));
        }
        break;
      case 'n':
        if(runDry) {
          numNormals++;
        } else {
          getNormal(string(line+3));
        }
        break;
      case 't':
        if(runDry) {
          numTexCoords++;
        } else {
          getTexCoord(string(line+3));
        }
        break;
      default:
        break;
      }
      break;
    case 'g':
      if(runDry) {
        numMeshFaces.push_back(0);
      } else {
        nMesh++;
        cout << "Got mesh: " << (line+2) << endl;
        meshes[nMesh].name = (line+2);
      }
      break;
    case 'f':
      if(runDry) {
        numMeshFaces.back()++;
      } else {
        getFace(string(line+2));
      }
      break;
    default:
      break;
    }
  }
  if(runDry) {
    cout << "Numbers: " << numVertices << " " << numNormals << " " << numTexCoords << endl;
    reserve(numVertices, numNormals, numTexCoords, numMeshFaces);
  }
  return true;
}

void Obj::getVertex(string str) {
  float x, y ,z;
  sscanf(str.c_str(), "%f %f %f", &x, &y, &z);
  vertices.push_back(GLVector(x, y, z));
}

void Obj::getNormal(string str) {
  float x, y ,z;
  sscanf(str.c_str(), "%f %f %f", &x, &y, &z);
  normals.push_back(GLVector(x, y, z));
}

void Obj::getTexCoord(string str) {
  TextureCoordinate t;
  sscanf(str.c_str(), "%f %f", &t.u, &t.v);
  textureCoords.push_back(t);
}

void Obj::getFace(string str) {
  string vertex;
  Face f;
  Mesh *cur = &meshes.back();
  unsigned int pos;
  do {
    Vertex v;
    pos = str.find_first_of(' ');
    vertex = str.substr(0,pos);
    if(!cur->gotFlags) { 
      cur->flags = getMeshFlags(vertex);
      cur->gotFlags = true;
    }
    int pos2 = vertex.find_first_of('/');
    sscanf(vertex.substr(0, pos2).c_str(), "%d", &v.vertexId);
    vertex = vertex.substr(pos2+1);
    if(cur->flags & (HAS_TEXTURE | HAS_NORMALS)) {
      pos2 = vertex.find_first_of('/');
      if(cur->flags & HAS_TEXTURE) {
        sscanf(vertex.substr(0,pos2).c_str(), "%d", &v.textureId);
      } else {
        v.textureId = 0;
      }
      vertex = vertex.substr(pos2+1);
      if(cur->flags & HAS_NORMALS) {
        sscanf(vertex.c_str(), "%d", &v.normalId);
      } else {
        v.normalId = 0;
      }
    } else {
      v.normalId = v.textureId = 0;
    }
    f.vertexIds.push_back(v);
    str = str.substr(pos+1);
  } while(pos != string::npos);
  cur->faces.push_back(f);
}

int Obj::getMeshFlags(string vertex) {
  int flags = 0;
  vertex = vertex.substr((vertex.find_first_of('/')+1));
  unsigned int pos = vertex.find_first_of('/');
  if((pos != string::npos) && (vertex.substr(0, pos).length() > 0)) {
    cout << "Has texture coords.." << endl;
    flags |= HAS_TEXTURE;
  }
  if((pos != string::npos) && (vertex.substr(pos+1).length() > 0)) {
    cout << "Has normals.." << endl;
    flags |= HAS_NORMALS;
  }
  return flags;
}

void Obj::render() {
  glPushMatrix();
  applyTransformation();
  for(unsigned int i = 0; i < meshes.size(); i++) {
    glColor3f(0.8, 0.8, 0.8);
    renderMesh(i);
  }
  glPopMatrix();
}

void Obj::renderMesh(unsigned int i) {
  if(meshes.size() >= i) {
    Mesh *m = &meshes[i];
    for (unsigned int c = 0; c < m->faces.size(); c++) {
      Face *f = &m->faces[c];
      bool vn = (m->flags & HAS_NORMALS);
      bool vt = (m->flags & HAS_TEXTURE);
      if(f->vertexIds.size() == 3) {
        glBegin(GL_TRIANGLES);
      } else if(f->vertexIds.size() == 4) {
        glBegin(GL_QUADS);
      } else {
        cerr << "Only triangles and quads are supported! Face: " << c << " Mesh: " << m->name << " Vertices: " << f->vertexIds.size() << endl;
        continue;
      }
      if(!vn) {
        GLVector n = computeNormal(vertices[f->vertexIds[0].vertexId-1], vertices[f->vertexIds[1].vertexId-1], vertices[f->vertexIds[2].vertexId-1], orientation);
        glNormal3f(n.x, n.y, n.z);
      }
      for(unsigned int j = 0; j < f->vertexIds.size(); j++) {
        Vertex *v = &f->vertexIds[j];
        if(vn) {
          GLVector *normal = &normals[v->normalId-1];
          glNormal3f(normal->x, normal->y, normal->z);
        }
        if(vt) {
          TextureCoordinate *texture =  &textureCoords[v->textureId-1];
          glTexCoord2f(texture->u, texture->v);
        }
        GLVector *vertex = &vertices[v->vertexId-1];
        glVertex3f(vertex->x, vertex->y, vertex->z);
      }
      glEnd();
    }
  } else {
    cerr << "No such mesh: " << i << endl;
  }
}
