// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste
// Changes copyright 2004 Kai Blin

#ifndef _GL_FILE_OBJ_H_
#define _GL_FILE_OBJ_H_

#include <string>
#include <vector>
#include <fstream>
#include <cstdio>
#include <GL/gl.h>
#include "GLVector.h"
#include "static_object.h"
#include <cassert>

using namespace std;

/**
 * A class taking care of static meshes of the OBJ type
 *
 *\author Henri Kuuste
 *\author Kai Blin (documentation)
 */
class Obj : public StaticObject {
 public:

  /**
   * Keep the texture coordinates together
   */
  typedef struct {
    float u, v;
  } TextureCoordinate;

  /**
   * Keep all the information about a vertex together
   */
  typedef struct {
    unsigned int  vertexId, textureId, normalId;
  } Vertex;

  /**
   * A face is a bunch of vertices
   */
  typedef struct {
    vector<Vertex> vertexIds;
  } Face;

  /**
   * Keep all the information about a mesh together
   */
  typedef struct {
    string        name;
    vector<Face>  faces;
    bool          gotFlags;
    int           flags;
  } Mesh;

  /**
   * Set some enum items
   */
  enum {
    HAS_TEXTURE = 1,
    HAS_NORMALS = 1 << 1,
  };

  /**   
   * Standard constructor, only sets the orientation
   */
  Obj();

  /**
   * Constructor giving a filename and a cal3d bone
   *
   *\param filename filename to load the object from
   *\param bone pointer to a CalBone, defaults to NULL
   */
  Obj(const std::string filename, CalBone *bone = NULL);

  /**
   * Standard destructor
   */
  ~Obj();

  /**
   * Read the OBJ file.
   *
   * If it's a dry run, just count the vertices, normals, textures and faces and reserve some
   * space for them to speed up loading later.
   * 
   *\param file std::string containing the filename
   *\param is this a dry run? Defaults to false
   *\return true on success, false otherwise
   */
  bool readFile(std::string file, bool runDry = false);

  /**
   * Reserve space for the vertices, normals, textures and faces to speed up loading
   *
   *\param numVertices the number of vertices to reserve space for
   *\param numNormals the number of normals to reserve space for
   *\param numTexCoords the number of texture coordinates to reserve space for
   *\param numMeshFaces the number of meshes and faces to reserve space for
   */
  void reserve(unsigned int numVertices, unsigned int numNormals, unsigned int numTexCoords, vector<unsigned int> &numMeshFaces);
  
  /**
   * Render the object, iterating over the meshes
   */
  void render();

  /**
   * Render a mesh
   * 
   *\param i number of mesh to render
   */
  void renderMesh(unsigned int i);

  /**
   * Get the mesh flags from a string
   *
   *\param vertex std::string to check
   *\return a bitmask with all flags for a mesh
   */
  int getMeshFlags(string vertex);

  /**
   * Load a vertex from a string
   *
   *\param str std::string to load vertex from
   */
  void getVertex(string str);

  /**
   * Load a normal from a string
   *
   *\param str std::string to load normal from
   */
  void getNormal(string str);

  /**
   * Load a texture coordinate from a string
   *
   *\param str std::string to load texture coordinate from
   */
  void getTexCoord(string str);

  /**
   * Load a face from a string
   *
   *\param str std::string to load face from
   */
  void getFace(string str);
	
  /**
   * A vector containing all meshes
   */
  vector<Mesh>               meshes;

  /**
   * A vector containing all vertices
   */
  vector<GLVector>           vertices;

  /**
   * A vector containing all normals
   */ 
  vector<GLVector>           normals;
  
  /**
   * A vector containing all texture coordinates
   */
  vector<TextureCoordinate>  textureCoords;

  /**
   * The orientation of the mesh
   */
  int                        orientation;
};

#endif
