// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste
// Changes copyright 2004 Kai Blin

#ifndef _EF_STATIC_OBJECT_H_
#define _EF_STATIC_OBJECT_H_

#include <cal3d/cal3d.h>
#include <GL/gl.h>
#include "efmath.h"
#include "arcball.h"
#include <gtkmm.h>
#include <sigc++/object.h>
#include <sigc++/object_slot.h>

/**
 * Base class of all types of static meshes
 *
 *\author Henri Kuuste
 *\author Kai Blin (documentation)
 */

class StaticObject : public sigc::trackable {
public:

  /**
   * Standard constructor, create mesh without a cal3d bone
   */
  StaticObject();

  /**
   * Create mesh with a cal3d bone
   *
   *\overload
   *\param bone pointer to a cal3d bone
   */
  StaticObject(CalBone *bone);

  /**
   * Standard destructor
   */
  virtual ~StaticObject();

  /**
   * Attach a cal3d bone to the mesh
   *
   *\param bone CalBone to attach
   */
  void attach(CalBone *bone);

  /**
   * Detach the cal3d bone from the mesh
   */
  void detach();
	
  /**
   * Set the translation of the mesh
   *
   *\param x x-axis translation
   *\param y y-axis translation
   *\param z z-axis translation
   */
  void setTranslation(double x, double y, double z);
		
  /**
   * Get the transformation for the x axis
   */
  double tx() { return txAd->get_value(); };
  
  /**
   * Get the transformation for the y axis
   */
  double ty() { return tyAd->get_value(); };

  /**
   * Get the transformation for the z axis
   */
  double tz() { return tzAd->get_value(); };

  /**
   * Set the scale individually for every axis
   *\overload
   *\param x factor to scale the x-axis by
   *\param y factor to scale the y-axis by
   *\param z factor to scale the z-axis by
   */
  void setScale(double x, double y, double z);

  /**
   * Set the same scale for every axis
   *
   *\param x factor to scale every axis by
   */
  void setScale(double x);


  /**
   * Get the scale for the x axis
   */
  double sx() { return sxAd->get_value(); };

  /**
   * Get the scale for the y axis
   */
  double sy() { return syAd->get_value(); };

  /**
   * Get the scale for the z axis
   */
  double sz() { return szAd->get_value(); };

  //	CalVector getCenter() { if(bone_) { return bone_->getTranslationAbsolute(); } else { return CalVector(); } };

  /**
   * Apply the transformation to the mesh, taking the cal3d bone into account if there is one
   */
  void applyTransformation();

  /**
   * Render the static mesh
   */
  virtual void render();

  /**
   * render the pivoted mesh
   */
  void renderPivot();

  /**
   * Create a setup dialog asking about the settings for a static mesh
   *
   *\param main parent window of the dialog
   */
  void createSetupDialog(Gtk::Window *main);

  /**
   * A GTK table with settings for scale and translation
   */
  Gtk::Table *createControlsTable();

  /**
   * Run the setup dialog
   */
  void setup();

  /**
   * Change the rotation using a quaternion
   *
   *\param q quaternion to rotate with
   */
  void adjustRotation(CalQuaternion q);

  /**
   * An arc ball to rotate the model if it doesn't have a root bone, I think
   *
   *\todo fix the documentation for this
   */
  ArcBall ball;
	
protected:
  /**
   * A cal3d bone to attach to the mesh
   */
  CalBone *bone_;
  /**
   * A quaternion to rotate the mesh
   */
  CalQuaternion qAdjust_;

  /**
   * A setup dialog for static meshes
   */
  Gtk::Dialog *dialog;
  /**
   * GTK::Adjustments for the transformation and scale axes
   */
  Gtk::Adjustment *txAd, *tyAd, *tzAd, *sxAd, *syAd, *szAd;
};

#endif
